--Speedometer.
--by aarbro [aarbro@hotmail.com]


Speedometer = {

	Editor = {
		Model="Objects/EXP/Editor/editor_speedo.cgf",
	},

	Properties = {
		VehicleName = "", --only active for a specific type or instance on the map. Empty for all.
		Units = "kph", --units for speed and tripmeter, kph,mph,kts (km/h, mi/h, knots)
		bEnabled = 1,

		Dial = {
			fileTexture = "Objects/EXP/Gauges/kph.dds", --dds/dxt1, one bit or no transparency only, no mipmaps needed.
			ColorR = 1,
			ColorG = 1,
			ColorB = 1,
			Transparency = 1,
			Blending = 4, --4 for transparency, 0 for opaque...
			LocationX=660,
			LocationY=50,
			Width=128,
			Height=128,
		},

		Needle = {
			CenterX = 64, --rotation center of the needle, offset from top left origin of dial texture.
			CenterY = 64,
			Length = 20, --total length in pixels
			ColorR = 1,
			ColorG = 1,
			ColorB = 1,
			Transparency = 1,
			fAngleMin = 90, --down is 0, left 90, up 180, right 270. Increasing clockwise.
			fAngleMax = 270, --can turn left or right but never pass 0.
			fSpeedMin = 20,
			fSpeedMax = 200,
			fExp = -1,
		},

		SpeedDigits = {
			bShowDigits = 0,
			DigitFormat = "%u", --format as printf
			FontName = "default",
			FontEffect = "default",
			Width = 20,
			Height = 20,
			LocationX = 64, --top left position, offset from top left origin of dial texture.
			LocationY = 128,
			ColorR = 1,
			ColorG = 1,
			ColorB = 1,
			Transparency = 1,
			Scale = 1.0,
		},

		TripDigits = {
			bShowDigits = 0,
			DigitFormat = "%07u", --format as printf
			FontName = "default",
			FontEffect = "default",
			Width = 20,
			Height = 20,
			LocationX = 64, --top left position, offset from top left origin of dial texture.
			LocationY = 128,
			ColorR = 1,
			ColorG = 1,
			ColorB = 1,
			Transparency = 1,
			Scale = 1.0,
		},
	},
	
	dial_texture = nil,
	unit_mult = 1.0,
	last_time = 0,
	distance = 0,
	
};
-------------------------------------------------------------------------------
function Speedometer:OnPropertyChange()
	self:OnReset();
end
-------------------------------------------------------------------------------

function Speedometer:OnInit()
	self:OnReset();

end
-------------------------------------------------------------------------------

function Speedometer:OnReset()


	self:EnableUpdate(0);
	
	self.dial_texture = System:LoadTexture( self.Properties.Dial.fileTexture );

	self.unit_mult = 1.0;
	if ( self.Properties.Units == "kph" ) then
		self.unit_mult = 3.6;
	elseif ( self.Properties.Units == "mph" ) then
		self.unit_mult = 2.236936;
	elseif ( self.Properties.Units == "kts" ) then
		self.unit_mult = 1.943844;
	end
if (Hud~=nil) then
	Hud:EnableCustomHud(1); --turn on global hud updates
end
	
	self.last_time = System:GetCurrTime();

end

-------------------------------------------------------------------------------
function Speedometer:OnUpdate()

end
-------------------------------------------------------------------------------
function Speedometer:OnShutDown()
end
-------------------------------------------------------------------------------
function Speedometer:OnSave(stm)
end
-------------------------------------------------------------------------------
function Speedometer:OnLoad(stm)
end
-------------------------------------------------------------------------------

function Speedometer:OnUpdateCustomHud()

	if ( self.Properties.bEnabled == 0 ) then
		return;
	end

	if ( not _localplayer.theVehicle ) then --player is not in vehicle
		return;
	end

	if ( self.Properties.VehicleName ~= _localplayer.theVehicle.classname and --wrong type of vehicle
			 self.Properties.VehicleName ~= _localplayer.theVehicle:GetName() and
			 self.Properties.VehicleName ~= "" ) then
		return;
	end

	if ( _localplayer.theVehicle.driverT.entity ~= _localplayer ) then --player is not driving
		return;
	end
	
	local speed = _localplayer.theVehicle.cnt:GetVehicleVelocity() * self.unit_mult;
	
	local time = System:GetCurrTime();
	local timedelta = time - self.last_time;
	local dist = timedelta / 360 * speed; --units (km, mile, nm) x 10
	self.distance = self.distance + dist;
	self.last_time = time;
	
	if ( self.Properties.Needle.fExp ~= -1 ) then
		speed = speed ^ self.Properties.Needle.fExp;
	end
	
	local amin = self.Properties.Needle.fAngleMin;
	local amax = self.Properties.Needle.fAngleMax;
	local smin = self.Properties.Needle.fSpeedMin;
	local smax = self.Properties.Needle.fSpeedMax;
	
	local ang_span = amax-amin;
	local spd_span = smax-smin;
	local ang = ( ( (limit( speed, smin, smax ) - smin) / spd_span) * ang_span ) + amin;
	
	ang = ( 360 - ang + 270 ) / 180 * PI; --move 0, change direction and convert to radians
	local x = cos( ang ) * self.Properties.Needle.Length;
	local y = -sin( ang ) * self.Properties.Needle.Length;
	local x0 = cos( ang + PI / 2 ) * 0.75;
	local y0 = -sin( ang + PI / 2 ) * 0.75;
	local x2 = cos( ang - PI / 2 ) * 0.75;
	local y2 = -sin( ang - PI / 2 ) * 0.75;
	
	local src_x = self.Properties.Dial.LocationX + self.Properties.Needle.CenterX;
	local src_y = self.Properties.Dial.LocationY + self.Properties.Needle.CenterY;
	local dest_x = self.Properties.Dial.LocationX + self.Properties.Needle.CenterX + x;
	local dest_y = self.Properties.Dial.LocationY + self.Properties.Needle.CenterY + y;

	
	System:DrawImageColor(
			self.dial_texture,
			self.Properties.Dial.LocationX,
			self.Properties.Dial.LocationY,
			self.Properties.Dial.Width,
			self.Properties.Dial.Height,
			self.Properties.Dial.Blending,
			self.Properties.Dial.ColorR,
			self.Properties.Dial.ColorG,
			self.Properties.Dial.ColorB,
			self.Properties.Dial.Transparency
		);


	System:Draw2DLine(
			src_x,
			src_y,
			dest_x,
			dest_y,
			self.Properties.Needle.ColorR,
			self.Properties.Needle.ColorG,
			self.Properties.Needle.ColorB,
			self.Properties.Needle.Transparency
		);
	System:Draw2DLine(
			src_x + x0,
			src_y + y0,
			dest_x,
			dest_y,
			self.Properties.Needle.ColorR,
			self.Properties.Needle.ColorG,
			self.Properties.Needle.ColorB,
			self.Properties.Needle.Transparency
		);
	System:Draw2DLine(
			src_x + x2,
			src_y + y2,
			dest_x,
			dest_y,
			self.Properties.Needle.ColorR,
			self.Properties.Needle.ColorG,
			self.Properties.Needle.ColorB,
			self.Properties.Needle.Transparency
		);

	if ( self.Properties.SpeedDigits.bShowDigits == 1 ) then
  	Game:SetHUDFont( self.Properties.SpeedDigits.FontName, self.Properties.SpeedDigits.FontEffect );
  	Game:WriteHudStringFixed( 
  		self.Properties.SpeedDigits.LocationX + self.Properties.Dial.LocationX,
  		self.Properties.SpeedDigits.LocationY + self.Properties.Dial.LocationY,
  		format( self.Properties.SpeedDigits.DigitFormat, speed),
  		self.Properties.SpeedDigits.ColorR,
  		self.Properties.SpeedDigits.ColorG,
  		self.Properties.SpeedDigits.ColorB,
  		self.Properties.SpeedDigits.Transparency,
  		self.Properties.SpeedDigits.Width,
  		self.Properties.SpeedDigits.Height,
  		self.Properties.SpeedDigits.Scale );
  end
		
	if ( self.Properties.TripDigits.bShowDigits == 1 ) then
  	Game:SetHUDFont( self.Properties.TripDigits.FontName, self.Properties.TripDigits.FontEffect );
  	Game:WriteHudStringFixed( 
  		self.Properties.TripDigits.LocationX + self.Properties.Dial.LocationX,
  		self.Properties.TripDigits.LocationY + self.Properties.Dial.LocationY,
  		format( self.Properties.TripDigits.DigitFormat, self.distance),
  		self.Properties.TripDigits.ColorR,
  		self.Properties.TripDigits.ColorG,
  		self.Properties.TripDigits.ColorB,
  		self.Properties.TripDigits.Transparency,
  		self.Properties.TripDigits.Width,
  		self.Properties.TripDigits.Height,
  		self.Properties.TripDigits.Scale );
  end

end
-------------------------------------------------------------------------------

function Speedometer:Event_Enable()
	BroadcastEvent(self, "Enable");
	self.Properties.bEnabled = 1;
end
-------------------------------------------------------------------------------
function Speedometer:Event_Disable()
	BroadcastEvent(self, "Disable");
	self.Properties.bEnabled = 0;
end
-------------------------------------------------------------------------------

function limit( val, vmin, vmax )
	if ( val < vmin ) then
		return vmin;
	end
	if ( val > vmax ) then
		return vmax;
	end
	return val;
end



